==> pandora_box-0.15.0.tar.gz: syd-pandora-0.15.0/src/caps/runtime.rs <==
/*!
Detect kernel features at runtime.

This module exposes methods to perform detection of kernel
features at runtime. This allows applications to auto-detect
whether recent options are implemented by the currently
running kernel.

## Example

```ignore
let ambient = caps::runtime::ambient_set_supported().is_ok();
println!("Supported ambient set: {}", ambient);

let all = caps::runtime::procfs_all_supported(None)
    .unwrap_or_else(|_| caps::runtime::thread_all_supported());
println!("Supported capabilities: {}", all.len());
```
!*/

use std::{
    io::Read,
    path::{Path, PathBuf},
};

use nix::errno::Errno;

use super::{ambient, CapSet, Capability, CapsHashSet};
use crate::{caps::errors::CapsError, err::err2no};

/// Check whether the running kernel supports the ambient set.
///
/// Ambient set was introduced in Linux kernel 4.3. On recent kernels
/// where the ambient set is supported, this will return `Ok`.
/// On a legacy kernel, an `Err` is returned instead.
pub fn ambient_set_supported() -> Result<(), CapsError> {
    ambient::has_cap(Capability::CAP_CHOWN)?;
    Ok(())
}

/// Return the set of all capabilities supported by the running kernel.
///
/// This requires a mounted `procfs` and a kernel version >= 3.2. By default,
/// it uses `/proc/` as the procfs mountpoint.
pub fn procfs_all_supported(proc_mountpoint: Option<PathBuf>) -> Result<CapsHashSet, CapsError> {
    /// See `man 2 capabilities`.
    const LAST_CAP_FILEPATH: &str = "./sys/kernel/cap_last_cap";
    let last_cap_path = proc_mountpoint
        .unwrap_or_else(|| PathBuf::from("/proc/"))
        .join(Path::new(LAST_CAP_FILEPATH));

    #[allow(clippy::disallowed_methods)]
    let max_cap: u8 = {
        let mut buf = String::with_capacity(4);
        std::fs::File::open(last_cap_path.clone())
            .and_then(|mut file| file.read_to_string(&mut buf))
            .map_err(|e| CapsError(err2no(&e)))?;
        buf.trim_end().parse().or(Err(CapsError(Errno::EINVAL)))?
    };

    let mut supported = super::all();
    for c in super::all() {
        if c.index() > max_cap {
            supported.remove(&c);
        }
    }
    Ok(supported)
}

/// Return the set of all capabilities supported on the current thread.
///
/// This does not require a mounted `procfs`, and it works with any
/// kernel version >= 2.6.25.
/// It internally uses `prctl(2)` and `PR_CAPBSET_READ`; if those are
/// unavailable, this will result in an empty set.
pub fn thread_all_supported() -> CapsHashSet {
    let mut supported = super::all();
    for c in super::all() {
        if super::has_cap(None, CapSet::Bounding, c).is_err() {
            supported.remove(&c);
        }
    }
    supported
}

==> pandora_box-0.16.0.tar.gz: syd-pandora-0.16.0/src/caps/runtime.rs <==
/*!
Detect kernel features at runtime.

This module exposes methods to perform detection of kernel
features at runtime. This allows applications to auto-detect
whether recent options are implemented by the currently
running kernel.

## Example

```ignore
let ambient = caps::runtime::ambient_set_supported().is_ok();
println!("Supported ambient set: {}", ambient);

let all = caps::runtime::procfs_all_supported(None)
    .unwrap_or_else(|_| caps::runtime::thread_all_supported());
println!("Supported capabilities: {}", all.len());
```
!*/

use std::{
    io::Read,
    path::{Path, PathBuf},
};

use nix::errno::Errno;

use super::{ambient, CapSet, Capabilities, Capability};
use crate::{caps::errors::CapsError, err::err2no};

/// Check whether the running kernel supports the ambient set.
///
/// Ambient set was introduced in Linux kernel 4.3. On recent kernels
/// where the ambient set is supported, this will return `Ok`.
/// On a legacy kernel, an `Err` is returned instead.
pub fn ambient_set_supported() -> Result<(), CapsError> {
    ambient::has_cap(Capability::CAP_CHOWN)?;
    Ok(())
}

/// Return the set of all capabilities supported by the running kernel.
///
/// This requires a mounted `procfs` and a kernel version >= 3.2. By default,
/// it uses `/proc/` as the procfs mountpoint.
pub fn procfs_all_supported(proc_mountpoint: Option<PathBuf>) -> Result<Capabilities, CapsError> {
    /// See `man 2 capabilities`.
    const LAST_CAP_FILEPATH: &str = "./sys/kernel/cap_last_cap";
    let last_cap_path = proc_mountpoint
        .unwrap_or_else(|| PathBuf::from("/proc/"))
        .join(Path::new(LAST_CAP_FILEPATH));

    #[allow(clippy::disallowed_methods)]
    let max_cap: u8 = {
        let mut buf = String::with_capacity(4);
        std::fs::File::open(last_cap_path.clone())
            .and_then(|mut file| file.read_to_string(&mut buf))
            .map_err(|e| CapsError(err2no(&e)))?;
        buf.trim_end().parse().or(Err(CapsError(Errno::EINVAL)))?
    };

    // Filter the library-known capabilities by kernel's max_cap index
    let supported = {
        // mask with bits [0, max_cap]
        let mask: u64 = if max_cap >= 63 {
            u64::MAX
        } else {
            (1u64 << (u64::from(max_cap) + 1)) - 1
        };

        super::Capabilities::all() & super::Capabilities::from_bits_truncate(mask)
    };

    Ok(supported)
}

/// Return the set of all capabilities supported on the current thread.
///
/// This does not require a mounted `procfs`, and it works with any
/// kernel version >= 2.6.25.
/// It internally uses `prctl(2)` and `PR_CAPBSET_READ`; if those are
/// unavailable, this will result in an empty set.
pub fn thread_all_supported() -> Capabilities {
    let mut supported = Capabilities::empty();

    for flag in Capabilities::all() {
        if let Ok(cap) = Capability::try_from(flag) {
            if super::has_cap(None, CapSet::Bounding, cap).unwrap_or(false) {
                supported |= flag;
            }
        }
    }

    supported
}
